/**
 * (C) Copyright 2005-2006 Hewlett-Packard Development Company, L.P.
 */

/****************************** Selection Management *************************/

// The following values MUST match those declared in VsSession.java!
var IS = ",";  // Item separator
var ES = ";";  // Entry separator
var RS = "/";  // Record separator
var SELECTION_FIELD = "selectionField"; // Name of hidden input field containing selection
var IMPLICIT_SELECTION_FIELD = "implicitSelectionField"; // Name of hidden input field containing implicit selection
var SELECTION_SERVLET = "setselection";
var ID_INDEX = 0;
var GUID_INDEX = 1;
var TYPES_INDEX = 2;
var LABEL_INDEX = 3;

/** If true then sendSelection() will be called every time selection changes  */
var persistentSelection = false;

/** Bulk update level. */
var selectionBulkUpdateLevel = 0;

/** True if selectionBulkUpdateLevel > 0 and there were some selection changes */
var selectionBeforeBulkUpdates;

/** Call this method with value=true if the page supports persistent selection */
function setPersistentSelection(value) {
    persistentSelection = value;
}

/** Begins bulk update mode */
function selectionBeginBulkUpdates() {
	if(selectionBulkUpdateLevel == 0)
		selectionBeforeBulkUpdates = selectionGetFieldValue();
	selectionBulkUpdateLevel++;
}

/** Ends bulk update mode */
function selectionEndBulkUpdates() {
	vsAssert(selectionBulkUpdateLevel > 0, "Extra selectionEndBulkUpdates()");
	if(--selectionBulkUpdateLevel == 0) {
		var selectionAfterBulkUpdates = selectionGetFieldValue();
		if(selectionBeforeBulkUpdates != selectionAfterBulkUpdates)
		    selectionChanged(null);
		selectionBeforeBulkUpdates = null;
	}
}

/** Return selection field value. Private */
function selectionGetFieldValue() {
	// Fire an event if necessary
    var field = document.getElementById(SELECTION_FIELD);
    vsAssert(field.value != null, "Selection field not set");
    return field.value;
}

/** Sets new value for selection field. Fires selection change event if necessary. Private. */
function selectionSetFieldValue(newValue, hint) {
    vsValidateArgs(["string", "string"]);
	vsAssert(newValue != null, "");
	// Fire an event if necessary
    var field = document.getElementById(SELECTION_FIELD);
	var oldValue = field.value;
	field.value = newValue;
    if(selectionBulkUpdateLevel == 0) {
		if(oldValue != newValue) {
			selectionChanged(hint);
		}
	}
}

var selectionChangedCalling = false;

function selectionChanged(hint) {
    selectionChangedCalling = true;
    try {
        selectionChangedNotifiers(null);
    } finally {
        selectionChangedCalling = false;
    }
    if(persistentSelection)
        sendSelection();
}

/**
 * Creates an entry for the selection list from an object id, guid, its array of types
 * and label.
 * See the internal comments in VsSession.java for the format of the entry.
 */
function selectionEntry(id, guid, types, label) {
    vsValidateArgs(["string", "string", "object", "string"]);
    // Build an IS-separated string of types (sort for repeatability)
    types = types.sort();
    var typeString = types.join(IS);
    var entry = id + ES + guid + ES + typeString + ES + label;
    return entry;
}

/**
 * Adds an entry to the selection list with the specified id, label and types.
 */
function selectionAdd(id, guid, types, label) {
    vsValidateArgs(["string", "string", "object", "string"]);
    var entry = selectionEntry(id, guid, types, label);
    selectionAddEntry(entry);
}

function selectionAddEntry(entry) {
    vsValidateArgs(["string"]);

    // Check if selection entry exists already
    // Retrieve the selection string, update it, store it back
    if(isSelectedEntry(entry))
		return;
    var selection = selectionGetFieldValue();
    var newSelection;
    if(selection == "") {
		newSelection = entry;
	} else {
		var entries = selection.split(RS);
		entries[entries.length] = entry;
		entries.sort();
		newSelection = entries.join(RS);
	}
    selectionSetFieldValue(newSelection, entry);

	// Update SIM selection
	var guid = entry.split(ES)[GUID_INDEX];
    var validSIMSelection = guid != "";
    if (validSIMSelection) {
        if (typeof(MxPortalNodesAddNode) != "undefined") {
            MxPortalNodesAddNode(guid);
        }
    } else {
        // No guid, thus there is at least one non HP SIM object in the selection (this one).
        // As we don't allow this type of heterogeneous selection in HP SIM, clear SIM selection
        if (typeof(MxPortalNodesClear) != "undefined") {
            MxPortalNodesClear();
        }
    }
}

/**
 * Sets the current selection to the contents of the passed string, which must
 * be in the encoded format that is stored
 */
function selectionSet(selection) {
    vsValidateArgs(["string"]);
	selectionBeginBulkUpdates();
	// remove entries that don't exist in new selection
	var oldEntries = selectionGetFieldValue().split(RS);
	var newEntries = selection.split(RS);
	for(var i = 0; i < oldEntries.length; i++) {
		if(!contains(newEntries, oldEntries[i]))
			selectionRemoveEntry(oldEntries[i]);
	}
	// add entries that exist in new selection but don't exist in old selection
	for(var i = 0; i < newEntries.length; i++) {
		selectionAddEntry(newEntries[i]);
	}
	selectionEndBulkUpdates();
}

function contains(list, item) {
	for(var i = 0; i < list.length; i++)
		if(list[i] == item)
			return true;
	return false;
}

/**
 * Removes the entry from the selection list with the specified id and types.
 * Pops an alert message if the entry is not found in the selection list.
 */
function selectionRemove(id, guid, types, label) {
    vsValidateArgs(["string", "string", "Array", "string"]);
    var entry = selectionEntry(id, guid, types, label);
    selectionRemoveEntry(entry);
}

function selectionRemoveEntry(entry) {
    vsValidateArgs(["string"]);
    // Removes the SelectedObject with the type and id from selectedObjects; if not found,
    // throws an alert
    var selection = selectionGetFieldValue();
    var selectedObjects = selection.split(RS);
    var found = false;
    var validSIMSelection = true;
    // Loop for all selected objects; when the specified entry is found, remove it
    for(var i = selectedObjects.length-1; i >= 0; i--) {
        var selectedObject = selectedObjects[i];
        if(selectedObject == entry) {
            // This removes a single element from the array
            selectedObjects.splice(i, 1);
            found = true;
        } else {
			var guid = selectedObject.split(ES)[GUID_INDEX];
			validSIMSelection = validSIMSelection && guid != "";
        }
    }
	if(!found)
		return;

	// Update selection field
    var newSelection = selectedObjects.join(RS);
	selectionSetFieldValue(newSelection, entry);

	// Update SIM selection
	var deletedGuid = entry.split(ES)[GUID_INDEX];
    if(deletedGuid != "") {
        if(typeof(MxPortalNodesRemoveNode) != "undefined") {
            MxPortalNodesRemoveNode(deletedGuid);
        }
    } else {
		if(validSIMSelection && typeof(MxPortalNodesAddNode) != "undefined" ) {
            // No missing guids, all remaining selections are SIM objects. Set HP SIM selection
            // to contain these objects. No need to clear HP SIM selection here, as it was
            // cleared when the non HP SIM object was selected.
            for(var i = 0; i < selectedObjects.length; i++ ) {
		        var selectedObject = selectedObjects[i];
				var guid = selectedObject.split(ES)[GUID_INDEX];
                MxPortalNodesAddNode(guid);
            }
        }
    }
}

/**
 * Get the value of the hidden field passed in and use it to remove the
 * entry from the selection list with the specified id.
 *
 * Calls to this function are generated by the RemoveSelectionIndirectNotifier
 * class.
 */
function selectionRemoveIndirect(hiddenFieldName) {
    vsValidateArgs(["string"]);
    var field = document.getElementById(hiddenFieldName);
    var selection = field.value;
        if ( selection != "" ) {
           selectionRemoveEntry(selection);
        }
}

function selectionClear() {
	selectionSetFieldValue("", null);
    if ( typeof(MxPortalNodesClear) != "undefined" ) {
        MxPortalNodesClear();
    }
}

/**
 * Returns true if the selection is not empty.
 */
function isSelectionEmpty() {
	var empty = (selectionGetFieldValue() == "")
    return empty;
}

/**
 * Returns true if the current selection contains exactly one item.
 */
function isSingleSelection() {
    var selection = selectionGetFieldValue();
    var selectedObjects = selection.split(RS);
    var singleSelection = (selectedObjects.length == 1);
    return singleSelection;
}

/**
 * Returns true if the selection is non-null AND all items on the selection list are of the
 * specified type.  Use min and max equal to or greater than zero to specify a range. 
 * A min or max less than zero will not be tested, e.g. to allow 2 or more selections:
 * set min=2, max=-1.  Min must be less than or equal to max.
 */
function isSelectionOfType(allowedTypes, min, max, exclusive) {
    vsValidateArgs(["Array", "int", "int", "boolean"]);
    vsAssert(allowedTypes != null && allowedTypes.length != 0,
                "isTypeSelected() argument '" + allowedTypes + "' must be non-null, non-empty");
    if (min >=0 && max >=0) {
        vsAssert( min <= max );
    }
    // An empty selection is of no type; it always returns false
    if ( isSelectionEmpty() ) { return false; }
    var selection = selectionGetFieldValue();
    var selectedObjects = selection.split(RS);
    var foundCount = 0;
    // Loop for all the objects in the selection list and check to see if they are of the
    // specified types; if any item is not of the specified types, this function returns false
    for ( var r = 0; r < selectedObjects.length; r++ ) {
        var selectedObject = selectedObjects[r];
        var objectAttributes = selectedObject.split(ES);
        var typeString = objectAttributes[TYPES_INDEX];
        var types = typeString.split(IS);
        var found = containsCommonValue(types, allowedTypes);
        if ( found ) { foundCount++; }

        // If exclusive is true, yet an object is selected that is not of the allowed type,
        // return false
        if ( exclusive && (! found) ) {
            return false;
        }
    }

    // If fewer than min items were found, return false.
    if ( min >= 0 && foundCount < min ) {
        return false;
    }

    // If more than max items were found, return false.
    if ( max >= 0 && foundCount > max ) {
        return false;
    }

    // If we get here, at least one selected object matches the allowed types, and
    // the exclusive and singleSelection issues have been taken care of. Return true!
    return true;
}

/**
 * Returns true if the two arrays have at least one element in common.
 */
function containsCommonValue(array1, array2) {
    for ( var i = 0; i < array1.length; i++ ) {
        for ( var j = 0; j < array2.length; j++ ) {
            if ( array1[i] == array2[j] ) {
                return true;
            }
        }
    }
    return false;
}

/**
 * Returns true if the specified id is on the selection list.
 */
function isSelected(id, guid, types, label) {
    vsValidateArgs(["string", "string", "Array", "string"]);
    vsAssert(id != null && id != "",
                "isSelected() argument '" + id + "' must be non-null, non-empty");
    // An empty selection always returns false
    if ( isSelectionEmpty() ) { return false; }
    var entry = selectionEntry(id, guid, types, label);
    return isSelectedEntry(entry);
}

function isSelectedEntry(entry) {
    vsValidateArgs(["string"]);
    var selection = selectionGetFieldValue();
    var selectedObjects = selection.split(RS);
    return contains(selectedObjects, entry);
}

/**
 * Returns comma separated string of selections of specified type
 * If checkImplicit is non zero, implicit selections will also
 * be considered.
 */
function getSelections(allowedTypes, checkImplicit) {
    vsValidateArgs(["Array", "boolean"]);
    vsAssert(allowedTypes != null && allowedTypes.length != 0,
             "allowedTypes argument '" + allowedTypes + "' must be non-null, non-empty");
    var selection = selectionGetFieldValue();
    var ifield = document.getElementById(IMPLICIT_SELECTION_FIELD);
    var iselection = "";
    if (ifield != null) {
        iselection = ifield.value;
    }
    var allSelection = "";
    if (checkImplicit) {
        if (iselection.length > 0 && selection.length > 0) {
            allSelection = iselection + RS + selection;
        } else {
            allSelection = iselection + selection;
        }
    } else {
        allSelection = selection;
    }
       if (allSelection.length == 0) { return ""; }
       var output = "";
    var selectedObjects = allSelection.split(RS);
    // Loop for all the objects in the selection list and check to see if
    // they are of the specified types; if any item is not of the specified
    // types, this function returns empty string
    for ( var r = 0; r < selectedObjects.length; r++ ) {
        var selectedObject = selectedObjects[r];
        var objectAttributes = selectedObject.split(ES);
        var typeString = objectAttributes[TYPES_INDEX];
        var types = typeString.split(IS);
        var found = containsCommonValue(types, allowedTypes);
        if (found) {
            if (output.length > 0) {
                output += ", ";
            }
            output += objectAttributes[LABEL_INDEX];
        }
    }
    return output;
}

var selectionChangedHint = null;

function updateSelectionHighlight(entry, gadgetid, background) {
    if ( selectionChangedHint != null && selectionChangedHint != entry ) {
        // We know what changed and it wasn't this entry. Don't do anything!
        return;
    }
    if ( isSelectedEntry(entry) ) {
        if ( background != null ) {
            /** @todo js Where does the selection color definition go? */
            var backgroundElement = document.getElementById(background);
                        if (backgroundElement != null) {
                                backgroundElement.style.backgroundColor='#99ccff';
                        }
        }
        if ( gadgetid != null ) {
            var gadget = document.getElementById(gadgetid);
                        if (gadget != null) {
                                gadget.checked=true;

                                // Get the appropriate hidden field for either the radio button
                                // or the checkbox, and then set the value of the hidden field.
                                // Note - there is a separate hidden field for each checkbox,
                                //        but a single hidden field for a radio button group.
                                if (gadget.type == "radio") {
                                        var hiddenField = document.getElementById(gadget.name + "_hiddenField");
                                        hiddenField.value = entry;
                                }
                                else {
                                        var hiddenField = document.getElementById(gadgetid + "_hiddenField");
                                        hiddenField.value = "on";
                                }

                /**
                 * @todo js This call to appendClassName is necessary to ensure that the
                 * row highlighting is accurate in all situations. GUIDO has a table manager
                 * (tableManager.js) that hooks itself into all onclick callbacks on checkboxes
                 * with the "rowselector" attribute. I patched the tableManager so that it
                 * will also call the application onclick callback as well (previously it
                 * usurped the onclick callback entirely; see tableManager.js). This was necessary
                 * to allow changing the checkbox state to affect the VSE selection. However,
                 * when the selection is changed programmatically in Javascript (as when the
                 * selection state is updated at onload time, or when Deselect All is called),
                 * this method needs to be sure that the row color gets updated as well.
                 * Note that this will result in two calls to appendClassName/removeClassName
                 * in the typical case when the user clicks on a checkbox, but both methods
                 * are idempotent, thus this will have no negative effects.
                 * <p>
                 * It would be nice to find a way to call TableManager.tableRowHighlightToggle(),
                 * rather than this kluge. However, that method takes an event object, which
                 * may be difficult to fabricate.
                 * <p>
                 * Finally, it may be desireable to change SystemBoxG to use the same 'rowselector'
                 * mechanism for highlighting, if possible. At best, we could then eliminate the
                 * backgroundTag argument to this class, and rely on GUIDO to maintain the row
                 * color (along with the kluge below). Second best would be to change this class
                 * to append/remove rowHighlight to/from the class of the tag passed in, and
                 * rely on CSS to recolor the row. This would eliminate the need for the highlight
                 * color to be hardcoded in this class. Don't know if the SystemBoxG table structure
                 * matches that needed by rowHighlight, though.
                 */
                                if ( gadget.getAttribute('rowselector') == 'yes' ) {
                                        appendClassName(gadget.parentNode.parentNode, 'rowHighlight');
                                };
                        }
        }
    }
    else {
        if ( background != null ) {
            var backgroundElement = document.getElementById(background);
                        if (backgroundElement != null) {
                                backgroundElement.style.backgroundColor='';
                        }
        }
        if ( gadgetid != null ) {
            var gadget = document.getElementById(gadgetid);
                        if (gadget != null) {
                                gadget.checked=false;

                                if (gadget.type == "checkbox") {
                                        var hiddenField = document.getElementById(gadgetid + "_hiddenField");
                                        hiddenField.value = "reset";
                                }

                /**
                 * @todo js See comments above.
                 */
                                if ( gadget.getAttribute('rowselector') == 'yes' ) {
                                        removeClassName(gadget.parentNode.parentNode, 'rowHighlight')
                                };
                        }
        }
    }
}

var request;

/**
 * Sends the current selection back to the server asynchronously, without
 * reloading the current page. This allows periodic page refreshes to occur
 * without losing the current selection. See also the Java classSelectionServlet,
 * which is the receiver of this asynchronous communication.
 */
function sendSelection() {
    // Assume a canonical URL such as "https://hostname:50000/vse/servletname..."
    var urlComponents = document.URL.split("/");
    var protocol = urlComponents[0];
    var host = urlComponents[2];
    var context = urlComponents[3];
    var url = protocol + "//" + host + "/" + context + "/" + SELECTION_SERVLET;

    var selection = selectionGetFieldValue();
    var parameters = SELECTION_FIELD + "=" + encodeURI(selection);

    var ifield = document.getElementById(IMPLICIT_SELECTION_FIELD);
    if ( ifield != null ) {
        var iselection = ifield.value;
        parameters += IMPLICIT_SELECTION_FIELD + "=" + encodeURI(iselection);
    }

    if (window.XMLHttpRequest) { // Mozilla
        request = new XMLHttpRequest();
        //request.onreadystatechange = handlerFunction; // Not needed
        request.open("POST", url, true);
        request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        request.send(parameters);
    }
    else if (window.ActiveXObject) { // Internet Explorer
        request = new ActiveXObject("Microsoft.XMLHTTP");
        if (request) {
            // request.onreadystatechange = handlerFunction; // Not needed
            request.open("POST", url, true);
            request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
            request.send(parameters);
        }
    }
}
